/*
 * Copyright (C) the original author or authors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.derquinse.common.jaxrs;

import java.io.IOException;
import java.io.OutputStream;
import java.lang.annotation.Annotation;
import java.lang.reflect.Type;

import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.MultivaluedMap;
import javax.ws.rs.ext.MessageBodyWriter;
import javax.ws.rs.ext.Provider;

import net.derquinse.common.io.MemoryByteSource;

import com.google.common.io.ByteSource;

/**
 * {@link ByteSource} message body writer.
 * @author Andres Rodriguez.
 */
@Provider
public final class ByteSourceBodyWriter implements MessageBodyWriter<ByteSource> {
	/** Default constructor. */
	public ByteSourceBodyWriter() {
	}

	@Override
	public boolean isWriteable(Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
		return ByteSource.class.isAssignableFrom(type);
	}

	@Override
	public long getSize(ByteSource t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType) {
		if (t instanceof MemoryByteSource) {
			return ((MemoryByteSource) t).size();
		}
		return -1;
	}

	@Override
	public void writeTo(ByteSource t, Class<?> type, Type genericType, Annotation[] annotations, MediaType mediaType,
			MultivaluedMap<String, Object> httpHeaders, OutputStream entityStream) throws IOException,
			WebApplicationException {
		t.copyTo(entityStream);
	}
}
